/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
<applet 
    CODEBASE=".."
    CODE="hamlet.AppButton.class" NAME="Plotter.Plotter"
    width=160 height=25 align=absbottom>
    <param name=LABEL value="Rod Heating Simulation">
    A Java applet should appear here
</applet>
*/


package Plotter;

import java.applet.Applet;
import java.awt.*;
import Plotter.*;
import hamlet.*;


public class Plotter extends AppFrame implements Runnable {
  //private declarations
  private Button step;
  private MenuItem start;
  private MenuItem stop;
  private Plotterfixed plot;
  private Rod r;
  private double a[];
  private double delta;
  private double diff;
  private int segs;
  private Thread animatorThread;
  private double elapsedTime = 0;
  
  private ScrollableNumbers lHeat;
  private ScrollableNumbers rHeat;
  private ScrollableNumbers initRod;
  private ScrollableNumbers length;
  private ScrollableNumbers td;
  private ScrollableNumbers numSegs;
  private Label deltaLabel;
  private NumericLabel elapsed;
  
  private final static int MAXSIZE = 1000;

  public Plotter()
  {
    a = new double[MAXSIZE];
    setLayout( new FlowLayout());
    
    /*set up the Animation menu */
    Menu animationMenu = new Menu("Animation");
    start = new MenuItem("Start");
    stop = new MenuItem("Stop");
    animationMenu.add(start);
    animationMenu.add(stop);
    menuBar.add(animationMenu);
    
   for (int i = 0; i <= 10; i++)
      a[i] = i*10;//Math.random() * 100;
      
	//Set up the text-entry kStuff
    animatorThread = null;
    Panel textPanel = new Panel();
    textPanel.setLayout(new GridLayout(8,1,0,5));
    lHeat = new ScrollableNumbers(70.0, 1.0, 10.0, 
				  -9.9, 999.9, 4, 3, 
				  "Left heat source (C)", this);
    textPanel.add(lHeat);
    
    rHeat = new ScrollableNumbers(50.0, 1.0, 10.0, 
				  -9.9, 999.9, 4, 3, 
				  "Right heat source (C)", this);
    textPanel.add(rHeat);
    
    initRod = new ScrollableNumbers(0.0, 1.0, 10.0, 
				    -9.9, 999.9, 4, 3, 
				    "Initial rod temperature (C)", this);
    textPanel.add(initRod);
    
    length = new ScrollableNumbers(10.0, 1.0, 10.0, 
				   -9.9, 999.9, 4, 3, 
				   "Rod length (cm)", this);
    textPanel.add(length);
    
    td = new ScrollableNumbers(1.75, 1.0, 10.0, 
			       -9.9, 999.9, 4, 3, 
			       "Thermal Diffusivity", this);
    textPanel.add(td);
    
    numSegs = new ScrollableNumbers(10, 1.0, 10.0, 
				    10.0, 100.0, 4, 3, 
				    "Number of segments", this);
    textPanel.add(numSegs);
    
    deltaLabel = new Label("Delta t: 0.00 sec");
    textPanel.add(deltaLabel);

    elapsed = new NumericLabel ("Elapsed Time: ", 0.00, Label.LEFT, "sec");
    textPanel.add(elapsed);
    
    add(textPanel);
    

    plot = new Plotterfixed(a, 11, 1);
    plot.repaint();
    add(plot);
    step = new Button("Step");
    add(step);
    r = new Rod(10);
    add(r);
	
    validate();
    pack();
    //set up the graph
    initRod(a, numSegs.getIntText(), lHeat.getDoubleText(),
	      rHeat.getDoubleText(), initRod.getDoubleText());
    helpText = 
      "This applet simulates the heat distribution in a rod, as\n"	+
      "discussed in Chapter 17 of \"Introduction to Scientific \n"	+
      "Programming\" by Joseph L. Zachary.\n\n"				+ 
      //								//
      "File Menu\n"							+
      "   + Reset:  Sets all rod values to their value at time 0\n"	+
      "   + Refresh: Refreshes the applet\n"				+
      "   + Hide:  Removes the applet from the display\n"		+
      "   + Quit:  Terminates the applet\n\n"				+
      //								//
      "Input Fields \n"							+ 
      "   + Left heat source: The temperature of the left heat source\n"+
      "   + Right heat source: The temperature of the right heat source\n"+
      "   + Initial rod temperature:  The initial temperature of the\n"	+
      "      rod\n"							+
      "   + Rod length: The length of the rod, in centemeters\n"	+
      "   + Thermal Diffusivity: The thermal diffusivity of the rod\n"  +
      "   + Number of Segments:  The number of segments we divide\n"	+
      "      the rod into for the simulation.\n\n"			+
      //								//
      "Output Field\n"							+
      "   + Delta t: The time between steps, in seconds.\n\n"		+
      //								//
      "Plot\n"								+
      "   A plot of the temperature distribution along the rod.  The\n"	+
      "   segment boundaries are plotted along the horizontal axis and\n"+
      "   temperatures plotted on the vertical axis.\n\n"		+
      //								//
      "Rod\n"								+
      "   A simulation of the rod, with colors representing the\n"	+
      "   temperature of each segment.  Black represents the coolest\n" +
      "   spots on the rod and pure red the hottest.\n"			+
      "";

  }

  public Dimension preferredSize()
  {
    return new Dimension(700,650);
  }

  public Dimension minimumSize()
  {
    return new Dimension(700,650);
  }
  
  public void reset()
  {
  	initRod(a, numSegs.getIntText(), lHeat.getDoubleText(),
	      rHeat.getDoubleText(), initRod.getDoubleText());
	animatorThread = null;
	step.enable();
	elapsedTime = 0;
  }
  
  //the actions of this function are under debate-- do we stop the animation while
  //the window is hidden or do we let it continue to run
  public void hide()
  {
  	animatorThread = null;
  	super.hide();
  }


  public boolean action(Event evt, Object what)
  {
    boolean handled = true;

    if (evt.target == step) {
      double deltaTime = ((length.getDoubleText() * length.getDoubleText()) /
			  (numSegs.getIntText() * numSegs.getIntText())) /
			    (2.0 * td.getDoubleText());
      nextRod(a, numSegs.getIntText());
      elapsedTime += deltaTime;
      elapsed.setNumber(elapsedTime);
    }
    else if (evt.target == start)
    {
      step.disable();
      begin();
    }
    else if (evt.target == stop)
    {
      step.enable();
      stop();
    }
    else
      handled = super.action(evt, what);
    return handled;
  } 
  
  public void initRod(double cur[], int segs, double left, double right,
		      double rod)
  {
    int i;
    
    String tempS = "Delta t: "
      + Double.toString(((length.getDoubleText() * length.getDoubleText()) /
			 (numSegs.getIntText() * numSegs.getIntText())) /
			(2 * td.getDoubleText())) + " sec";
    deltaLabel.setText(tempS);
    cur[0] = left;
    for (i = 1; i < segs; i++)
      cur[i] = rod;
    cur[segs] = right;
    
    plot.reset(a,segs+1,1);
    plot.repaint();
    r.SetPoints(a, segs+1);
    r.repaint();
  }
  
  /* Figure out what the rod should look like at the next time
     segment. This is done by making it an average between 
     the segment in front and the segment in back of the
     current rod. */
  public void nextRod(double cur[], int segs)
  {
    double nxt[] = new double[segs+1];
    int i;
    
    for (i = 1; i < segs; i++)
      nxt[i] = (cur[i-1] + cur[i+1]) / 2;
      
    //System.out.print(cur[0] + "  ");
    for (i = 1; i < segs; i++)
    {
    //  System.out.print(nxt[i] + "  ");
      cur[i] = nxt[i];
    }
    //System.out.println(cur[segs]);
    
    plot.resetpoints(a,segs+1);
    plot.repaint();
    r.SetPoints(a,segs);
    r.repaint();
  }

  /* Animation kStuff */
  public void stop()
  {
    animatorThread = null;
  }

  public void begin() {
    if (animatorThread == null){
      animatorThread = new Thread(this);
      animatorThread.start();
    }
  }
  
  public void run() {
  
    plot.setPaintMethod((short) 1);	//we want to draw only the 
					//points while running an animation
    
    //Just to be nice, lower this thread's priority
    //so it can't interfere with other processing going on.
    Thread.currentThread().setPriority(Thread.MIN_PRIORITY);

    //Remember the starting time.
    long startTime = System.currentTimeMillis();
    double deltaTime = ((length.getDoubleText() * length.getDoubleText()) /
			 (numSegs.getIntText() * numSegs.getIntText())) /
			(2.0 * td.getDoubleText());
        
    //This is the animation loop.
    while (Thread.currentThread() == animatorThread) {
      
      //Display it.
      nextRod(a, numSegs.getIntText());
      elapsedTime += deltaTime;
      elapsed.setNumber(elapsedTime);

      //Delay depending on how far we are behind.
      try {
		startTime += delta;
		Thread.sleep(20);
      } catch (InterruptedException e) {
		break;
      }
    }
    plot.setPaintMethod((short) 0); //set the draw method back 
				    //to the original painting method
  }
}
