/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package Plotter;

import java.awt.*;

class Rod extends Canvas
{
  private final static int MAXSIZE = 1000;
  private final static int frame_width = 350;
  private final static int frame_height = 25;
  
  private double a[];
  private int numPoints;
  private Dimension prefSize;
  
  /* Set up a new Rod with the given number of segements */
  public Rod(int num)
  {
    a = new double[MAXSIZE];
    numPoints = num;
    prefSize = new Dimension(frame_width, frame_height);
  }

  /* Set up a Rod with the initial values of d and size
   * num. */
  public Rod(double d[], int num)
  {
    SetPoints(d, num);
    prefSize = new Dimension(frame_width, frame_height);
  }
  
  /* What is the maximum temperature on the rod? */
  public double max(double d[])
  {
    double temp = d[0];
    int i = 0;
    
    for (i = 0; i < numPoints; i++)
      if (temp < d[i])
	temp = d[i];
    return temp;
  }

  /* What is the minimum temperature on the rod? */
  public double min(double d[])
  {
    double temp = d[0];
    int i = 0;
    
    for (i = 0; i < numPoints; i++)
      if (temp > d[i])
	temp = d[i];
    return temp;
  }
  
  /* Given an array, set the Rod's internal array to match */
  public void SetPoints(double d[], int num)
  {
    for (int i = 0; i < num; i++)
      a[i] = d[i];
    numPoints = num;
  }
  
  public void update(Graphics g)
  {
  	paint(g);
  }

  public void paint(Graphics g)
  {
    double scale = 255.0 / max(a);
    Color c;
    int xdelta = size().width / numPoints;
    
    for (int i = 0; i < numPoints; i++){
      c = new Color(((int) Math.round(a[i] * scale)), 0, 0);
      g.setColor(c);
      g.fillRect(i*xdelta, 0, xdelta,size().height);
    }
  }

  public Dimension minimumSize() {
    return prefSize;
  }
  
  public Dimension preferredSize() {
    return prefSize;
  }
}
